""" plugin_dbui.py

    Controllers expose by the plugin:
    
        index
        database
        configuration

    Author: R. Le Gac
    $Id$

"""

__version__ = "$Revision$"


import os
import traceback
import sys


from gluon.contrib import simplejson as json
from gluon.http import HTTP
from gluon.tools import PluginManager
from pprint import pprint


SCRIPT = """
Ext.namespace('App');
App.name = '%s';
App.REMOTE_API = {
    'url': '/%s/plugin_dbui/call',
    'type': 'remoting',
    'actions': %s 
};
App.storeCfgs = %s;
App.withMathJax = %s; """


def call():
    """Controller to handle the Ext.Direct protocol.
    
    """
    return directSvc()


def debug():
    """Main Controller to run the plugin in debug mode
    Load the javascript source code for all libraries.
    Active the debug mode on the server side.

    EXAMPLES
    
        http://localhost:8000/myapp/plugin_dbui/debug
        http://localhost:8000/myapp/plugin_dbui/debug?script=foo

    PUBLIC PARAMETERS WHICH CAN BE SET VIA THE PLUGINMANAGER
        
        script_path
            The relative path where scripts are located.
            By default static/plugin_dbui/scripts
            
        user_src_path
            the relative path to a directory containing additional
            javascript source code for user application classes and plugins. 
            By default static/src
        
        
    OPTIONS

        script foo
            run the javascript foo in the framework defined by the plugin.
            The scripts are stored in the directory application/static/script 
            by default the javascript application/static/plugin_dbui/main.js 
            is launched.


    """
    # check the presence of the ExtJS plugin
    dbui = local_import('plugin_dbui')
    pextjs = dbui.get_extjs_path(globals())

    # internationalization (fr, fr-fr, fr-ca, ...)
    lg = dbui.get_language(globals())

    # dbui javascript source code -- respect the alphabetic order
    server_path, client_path = dbui.get_reference_paths(globals())
    
    directory = os.path.join('static', 'plugin_dbui', 'src')
    jsfiles = dbui.get_js_files(server_path, client_path, directory)
    response.files.extend(jsfiles)
    
    # user javascript source code -- respect the alphabetic order
    directory = plugins.dbui.user_src_path
    jsfiles = dbui.get_js_files(server_path, client_path, directory)
    response.files.extend(jsfiles)

    # main script
    pscript = dbui.get_script_path(globals())
    response.files.append(pscript)

    # switch ON the debug mode on the server side
    session.debug = True
    
    # return the ExtJS path, the language 
    # and a list of files in response.files
    return dict(pextjs=pextjs, lg=lg)
    
    

def get_api():
    """Controller returning the javascript API definition.
    
    """
    di = {}
    dbui = local_import('plugin_dbui')
    
    # build the dictionary required by Ext.direct
    # containing the definition of the remote procedure
    for (k, f) in directSvc.procedures.iteritems():
        action, method = k.split('.')

        if action == dbui.DBUI:
            nargs = f.func_code.co_argcount
        else:
            nargs = f.__func__.func_code.co_argcount - 1
            
        if action not in di:
            di[action] = []
            
        di[action].append({'name': method, 'len': nargs})
  
    # the stores configuration
    storeCfgs = {}
    for table in cfgSvc.get_tables()['tables']:
        storeCfgs[table] = cfgSvc.get_jsonstore(table)
        
    # determine if the MathJax plugin is there
    server_path, client_path = dbui.get_reference_paths(globals())
    path = os.path.join(server_path, 'static', 'plugin_mathjax')
    withMathJax = (True if os.path.exists(path) else False)

    # fill the javascript template
    app = request.application
    script = SCRIPT % (app, 
                       app, 
                       json.dumps(di),
                       json.dumps(storeCfgs), 
                       str(withMathJax).lower())
    
    # return the response as a javascript
    response.headers['Content-Type'] = 'text/javascript'
    return script
    
    
def index():
    """Default Controller to run the plugin
    Load compressed version of the ExtJS and user libraries.
    
    EXAMPLES
        
        http://localhost:8000/myapp/plugin_dbui
        http://localhost:8000/myapp/plugin_dbui?script=foo
        

    PUBLIC PARAMETERS WHICH CAN BE SET VIA THE PLUGINMANAGER
        
        script_path
            The relative path where scripts are located.
            By default static/plugin_dbui/scripts
                    
        user_libmin
            The relative path to the file containing the additional
            javascript library for user application classes and plugins. 
            It is the minified version of the source code.
            
        
    OPTIONS

        script foo
            run the javascript foo in the framework defined by the plugin.
            The scripts are stored in the directory application/static/script 
            by default the javascript application/static/plugin_dbui/main.js 
            is launched.

    """
    # check the presence of the ExtJS plugin
    dbui = local_import('plugin_dbui')
    pextjs = dbui.get_extjs_path(globals())
    
    # internationalization (fr, fr-fr, fr-ca, ...)
    lg = dbui.get_language(globals())

    # compressed version of the user library
    server_path, client_path = dbui.get_reference_paths(globals())
    
    if plugins.dbui.user_libmin:
        path = os.path.join(server_path, plugins.dbui.user_libmin)
        if os.path.exists(path):
            libmin = os.path.join(client_path, plugins.dbui.user_libmin)
            response.files.append(libmin)

    # main script
    pscript = dbui.get_script_path(globals())
    response.files.append(pscript)

    # switch off the debug mode on the server side
    session.debug = False

    # return the ExtJS path, the language 
    # and a list of files in response.files
    return dict(pextjs=pextjs, lg=lg)


def status():
    return dict(request=request, response=response, session=session)