""" $Id$

"""

__version__ = "$Revision$"
__author__ = "R. Le Gac"


from gluon.storage import Storage
from modifier import Modifier


GRID_MODIFIERS = 'grid_modifiers'

MSG_EXTJS_MISSING = 'keyword argument extjs is missing'
MSG_HEADER_MISSING = 'configuration option header is missing in extjs'
MSG_INVALID_KEYWORD = 'invalid keyword %s'
MSG_POSITION_MISSING = 'keyword argument position is missing'
MSG_TPL_MISSING = 'configuration option tpl is missing in extjs'


def configure_grids(db, **extjs):
    """Helper functions to configure all grid widgets with the same 
    configuration options. Useful to set plugins for example.
    
    The keyword arguments contain the configuration options for
    the Ext.grid.GridPanel.
    
    This tool can be use to set grid plugins for all grids in one go:
    > configure_grids(db, plugins=['pGridRowEditorContextMenu'])
    
    """
    for tablename in db.tables:
        gm = GridModifier(tablename)
        gm.configure(**extjs)


class GridModifierException(BaseException): pass

    
class GridModifier(Modifier):
    """Helper tool to customize the grid widget associated to a table.
    
    """
    
    def __init__(self, tablename):
        """Initialize the modifier persistent data.
        
        """
        Modifier.__init__(self, GRID_MODIFIERS, tablename)
        
        self.data.delete_columns = []
        self.data.grid_filters = []
        self.data.hidden_columns = []
        self.data.template_columns = []

                   
    def add_column(self, ktable, kfield, position=0, extjs={}):
        """Add a column displaying the database field kfield of the 
        foreign table ktable. 
        
        The keyword argument position is a number defining where the
        column will be insert. The first column is at 0.
        
        The keyword argument ExtJS is a dictionary containing
        configuration options of the underlying Ext.grid.Column.
        
        """


    def delete_columns(self, *fields):
        """Delete the columns in the Ext.grid.GridPanel widget.
        Each column is identified by its database field name.
        
        """
        for el in fields:
            if el not in self.data.delete_columns:
                self.data.delete_columns.append(el)

        
    def hide_columns(self, *fields):
        """Hide the columns list in arguments
        Each column is identified by its database field name.
        
        """
        for el in fields:
            if el not in self.data.hidden_columns:
                self.data.hidden_columns.append(el)
        
        
    def merge_columns(self, *fields, **kwargs):
        """Merge columns in a single template column Ext.grid.TemplateColumn.
        Each column is identified by its database field name.
        
        The new column should have an header and a template defining how the 
        merge field are displayed. These two configuration options of the 
        Ext.grid.Templatecolumn have to be defined via the keyword argument 
        extjs. The corresponding key of this dictionary are header and tpl.
        
        The syntax of the template is rather tricky to manipulate. 
        see the documentation of Ext.XTemplate constructor.        
        There is some limitation when defining the template in python.
        For example the use of the template member function is impossible.
        In the template, database fields are identified using the CapWord
        syntax: TablenameFieldName. The first letter of the tablename and field
        name are in upper case while all the other letters are in lower case.
        
        When the keyword argument autohide is set to true the columns in field
        are hidden.
        
        The keyword argument position is a number defining where the
        column will be insert. The first column is at 0.
        
        The keyword arguments extjs defines the configuration option
        of the underlying Ext.grid.TemplateColumn
        
        """
        keywords = ['autohide', 'extjs', 'position']

        for el in keywords:
            if el not in keywords:
                raise GridModifierException(MSG_INVALID_KEYWORD % el)
        
        if 'position' not in kwargs:
            raise GridModifierException(MSG_POSITION_MISSING)
        
        if 'extjs' not in kwargs:
            raise GridModifierException(MSG_EXTJS_MISSING)
        
        if 'header' not in kwargs['extjs']:
            raise GridModifierException(MSG_HEADER_MISSING)

        if 'tpl' not in kwargs['extjs']:
            raise GridModifierException(MSG_TPL_MISSING)
            
        di = Storage(extjs=kwargs['extjs'],
                     position=kwargs['position'])
        
        self.data.template_columns.append(di)
        
        if 'autohide' in kwargs and kwargs['autohide']:
            self.hide_columns(*fields)


    def set_filters(self, *filters, **kwargs):
        """Define the filters associated to the grid.
        The widget associated to each filter is a field, comboBox, dataField,..
        They are grouped in a Ext.form.Fieldset appearing in a panel on the 
        right side of the grid.
        
        A filter is define by a tuple containing three columns:
            - a string with the database field name
            - a string with the operator. Valid operator are defined in
              the method dbsvc._encode_query
            - a string with a comment for the tool tip
            
        The keyword argument extjs is a dictionary containing 
        the configuration options of the underlying Ext.form.FieldSet.

        """
        if 'extjs' not in kwargs:
            kwargs['extjs'] = {}
            
        self.data.grid_filters = Storage(filters=filters, extjs=kwargs['extjs'])
